%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%  - Main Program to segment the alveolus in healthy edentate patients -  %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
clc; clear all; close all;

% Add Lib Path
addpath("Lib");

% Clear GPU
reset(gpuDevice(1));
%% Parameters
options = struct;

% Heightmap parameters
options.heightmapXResolution = 1000;    % Points in total
options.heightmapYResolution = 1000;    % Points in total

% Raycast
options.rayCastLength = 5;              % Length of the perp lines in mm
options.estimationPoints = 100;         % Points in each perp line

% Other
options.resolution = 10;                % Points per milimeter

% Directories
options.startDirectory = ''; % Default opoen location
saveLocation = ''; % Default save location

%% Variable Instantation

% Scan info
scanInfo = struct;
scanInfo.mandibulaIncluded = false;

% Handle list
handles = struct;

% Results
results = struct;

% Results
coords = struct;

%% -- Start program


%% - Load Data

% Get the file name of the stl model
[stlFileName, stlFilePath] = uigetfile('*.stl','Select a maxilla stl file', options.startDirectory);

% Get the belonging scan info
fileParts = strsplit(stlFilePath, '\\');
scanInfo.patientFolder = fileParts{end-3};

xmlData = xml_read([strjoin(fileParts(1:end-2),'/') '/OrthoModelSetInfo.xml']);

% Get the part of the xml where the patient data is stored
tempScanInfo = parseScan(xmlData.Object.Property);

% Update the scanInfo
if(tempScanInfo.bAttachedLower == "True")
    scanInfo.mandibulaIncluded = true;
end

% Get the time of the plastercast
%totalIdentifier = tempScanInfo.wsModelSetExternalID;
totalIdentifier = tempScanInfo.wsModelSetID;
idParts = strsplit(totalIdentifier,'-');

if(length(idParts) < 2)
   error("Age is not correct"); 
end

if(length(idParts) == 3)
	scanInfo.month = str2double(idParts{2});
	scanInfo.year = str2double(idParts{1});
	scanInfo.day = str2double(idParts{3});
else
	scanInfo.month = str2double(idParts{2});
	scanInfo.year = str2double(idParts{1});
	scanInfo.day = 0;
end

% Load the model
model = stlread([stlFilePath '/' stlFileName]);

% Rotate model so the places corrected
if(scanInfo.mandibulaIncluded)
    model.vertices = rotateSTL(model.vertices, 0, -90, 0);
else
    model.vertices = rotateSTL(model.vertices, 0, 90, 180);
end

% Instantiate the opcodemesh
model.opcodemesh = opcodemesh(model.vertices',model.faces');

% Clear the vars which are not needed anymore
clear fileParts xmlData tempScanInfo idParts totalIdentifier;

%% Show the heightmap

% Update the resolution

[model.minValues, model.maxValues] = getBoundingBoxVertices(model.vertices);
diffValues = [abs(model.maxValues.x - model.minValues.x); abs(model.maxValues.y - model.minValues.y); abs(model.maxValues.z - model.minValues.z)];

if(diffValues(1) > diffValues(2))
    scaleValue = diffValues(2) / diffValues(1);
    options.heightmapYResolution = ceil(options.heightmapXResolution * scaleValue);
else
    scaleValue = diffValues(1) / diffValues(2);
    options.heightmapXResolution = ceil(options.heightmapYResolution * scaleValue);
end

% Calculate the heightmap
model.heightmap = makeHeightmapVertices(model.faces, model.vertices, options.heightmapXResolution, options.heightmapYResolution);

% Make a gradient of the heightmap
[model.gMagHeightmap, ~] = imgradient(model.heightmap);

% Show the gradient
handles.gradientMap = figure;
handles.gradientMapContent = imshow(model.gMagHeightmap,[-0.5, 1]);

% Create a continue button
btn = uicontrol('Style', 'pushbutton', 'String', 'Continue',...
    'Position', [20 20 50 20],...
    'Callback', 'close');

% Figure to second screen
set_figure_toscreen(2);
%imcontrast(gca);

% Show the figure patch
handles.patchMap = figure;
showPatch(model.faces, model.vertices, [0.6,0.6,0.6]);
camlight('left');
camlight('right');
set_figure_toscreen(1);

% Add the impoints
figure(handles.gradientMap);


title('Set Tuber Left'); handles.t1 = impoint(gca); setColor(handles.t1, 'b');
title('Set Tuber Right'); handles.t2 = impoint(gca); setColor(handles.t2, 'b');
title('Set Cuspid Left'); handles.c1 = impoint(gca); setColor(handles.c1, 'g');
title('Set Cuspid Right'); handles.c2 = impoint(gca); setColor(handles.c2, 'g');
title('Set Frontal Left'); handles.a1 = impoint(gca); setColor(handles.a1, 'y');
title('Set Frontal Right'); handles.a2 = impoint(gca); setColor(handles.a2, 'y');
title('Set End Segment Left'); handles.s1 = impoint(gca); setColor(handles.s1, 'r');
title('Set End Segment Right'); handles.s2 = impoint(gca); setColor(handles.s2, 'r');

title('Check the points with the 3D model');

% Update the 3D model
figure(handles.patchMap);
posMoved(1);

% Add callbacks
addNewPositionCallback(handles.t1, @posMoved); addNewPositionCallback(handles.t2, @posMoved);
addNewPositionCallback(handles.c1, @posMoved); addNewPositionCallback(handles.c2, @posMoved);
addNewPositionCallback(handles.a1, @posMoved); addNewPositionCallback(handles.a2, @posMoved);
addNewPositionCallback(handles.s1, @posMoved); addNewPositionCallback(handles.s2, @posMoved);

waitfor(handles.gradientMap);

% Move the 3D viewer
set_figure_toscreen(2);


%% Restart

if(~exist([saveLocation scanInfo.patientFolder],'dir'))
    mkdir([saveLocation scanInfo.patientFolder]);
end
model = rmfield(model, 'opcodemesh');
save([saveLocation scanInfo.patientFolder '/' num2str(scanInfo.year) '-' num2str(scanInfo.month) '-' num2str(scanInfo.day) '.mat']);

% Scandata
ULCP_Landmark_Step3

